import { AccountAddress } from './types/accountAddress';
import { ReduceStakePendingChange, RemovalPendingChange, AccountInfo, AccountInfoBaker, AccountInfoDelegator, GenerateBakerKeysOutput } from './types';
/** Whether {@link AccountInfo} parameter given is of type {@link AccountInfoDelegator}, i.e. the account is a delegator */
export declare const isDelegatorAccount: (ai: AccountInfo) => ai is AccountInfoDelegator;
/** Whether {@link AccountInfo} parameter given is of type {@link AccountInfoBaker}, i.e. the account is a baker. */
export declare const isBakerAccount: (ai: AccountInfo) => ai is AccountInfoBaker;
/** Whether the pending change given is of type {@link ReduceStakePendingChange} */
export declare const isReduceStakePendingChange: (spc: ReduceStakePendingChange | RemovalPendingChange) => spc is ReduceStakePendingChange;
/** Whether the pending change given is of type {@link RemovalPendingChange} */
export declare const isRemovalPendingChange: (spc: ReduceStakePendingChange | RemovalPendingChange) => spc is RemovalPendingChange;
/**
 * Generates random baker keys for the specified account, that can be used with the configureBaker transaction
 * @param account the address of the account that the keys should be added to.
 * @returns an object containing the public baker keys, their associated proofs and their associated private keys.
 */
export declare function generateBakerKeys(account: AccountAddress): GenerateBakerKeysOutput;
