export interface StatementBuilder<ValueType, AttributeType> {
    addRange(attribute: AttributeType, lower: ValueType, upper: ValueType): this;
    addMembership(attribute: AttributeType, set: ValueType[]): this;
    addNonMembership(attribute: AttributeType, set: ValueType[]): this;
    revealAttribute(attribute: AttributeType): this;
    getStatement(): GenericAtomicStatement<AttributeType, ValueType>[];
}
export declare const MIN_DATE = "18000101";
export declare const MAX_DATE = "99990101";
export declare const EU_MEMBERS: string[];
export declare enum StatementTypes {
    RevealAttribute = "RevealAttribute",
    AttributeInSet = "AttributeInSet",
    AttributeNotInSet = "AttributeNotInSet",
    AttributeInRange = "AttributeInRange"
}
declare type LaxStringEnum<E extends string> = `${E}`;
export declare type GenericRevealStatement<TagType> = {
    type: LaxStringEnum<StatementTypes.RevealAttribute>;
    attributeTag: TagType;
};
export declare type GenericMembershipStatement<TagType, ValueType> = {
    type: LaxStringEnum<StatementTypes.AttributeInSet>;
    attributeTag: TagType;
    set: ValueType[];
};
export declare type GenericNonMembershipStatement<TagType, ValueType> = {
    type: LaxStringEnum<StatementTypes.AttributeNotInSet>;
    attributeTag: TagType;
    set: ValueType[];
};
export declare type GenericRangeStatement<TagType, ValueType> = {
    type: LaxStringEnum<StatementTypes.AttributeInRange>;
    attributeTag: TagType;
    lower: ValueType;
    upper: ValueType;
};
export declare type GenericAtomicStatement<TagType, ValueType> = GenericRevealStatement<TagType> | GenericMembershipStatement<TagType, ValueType> | GenericNonMembershipStatement<TagType, ValueType> | GenericRangeStatement<TagType, ValueType>;
export declare type RevealProof<ValueType> = {
    type: StatementTypes.RevealAttribute;
    proof: string;
    attribute: ValueType;
};
export declare type GenericAtomicProof = {
    type: Exclude<StatementTypes, StatementTypes.RevealAttribute>;
    proof: string;
};
export declare type AtomicProof<ValueType> = RevealProof<ValueType> | GenericAtomicProof;
export {};
