import { AttributeKey, CryptographicParameters, IdentityObjectV1, Network, Versioned } from '.';
import { AtomicProof, GenericAtomicStatement, GenericMembershipStatement, GenericNonMembershipStatement, GenericRangeStatement, GenericRevealStatement } from './commonProofTypes';
export declare type RangeStatement = GenericRangeStatement<AttributeKey, string>;
export declare type NonMembershipStatement = GenericNonMembershipStatement<AttributeKey, string>;
export declare type MembershipStatement = GenericMembershipStatement<AttributeKey, string>;
export declare type RevealStatement = GenericRevealStatement<AttributeKey>;
export declare type AtomicStatement = GenericAtomicStatement<AttributeKey, string>;
export declare type IdStatement = AtomicStatement[];
export declare type IdProofInput = {
    idObject: IdentityObjectV1;
    globalContext: CryptographicParameters;
    seedAsHex: string;
    net: Network;
    identityProviderIndex: number;
    identityIndex: number;
    credNumber: number;
    statement: IdStatement;
    challenge: string;
};
export declare type IdProof = {
    proofs: AtomicProof<string>[];
};
export declare type IdProofOutput = {
    credential: string;
    proof: Versioned<IdProof>;
};
/**
 * The attributes that can be used for range statements
 */
export declare const attributesWithRange: AttributeKey[];
/**
 * The attributes that can be used for (non)membership statements
 */
export declare const attributesWithSet: AttributeKey[];
