import { GenericAtomicStatement, GenericRevealStatement, GenericMembershipStatement, GenericNonMembershipStatement, GenericRangeStatement } from './commonProofTypes';
import { ContractAddress, CryptographicParameters } from './types';
export declare type TimestampAttribute = {
    type: 'date-time';
    timestamp: string;
};
export declare type AttributeType = string | bigint | TimestampAttribute;
export declare type StatementAttributeType = AttributeType | Date;
export declare function isTimestampAttribute(attribute: AttributeType): attribute is TimestampAttribute;
export declare type AccountCommitmentInput = {
    type: 'account';
    issuer: number;
    values: Record<string, string>;
    randomness: Record<string, string>;
};
export declare type Web3IssuerCommitmentInput = {
    type: 'web3Issuer';
    signature: string;
    signer: string;
    values: Record<string, AttributeType>;
    randomness: Record<string, string>;
};
export declare type CommitmentInput = AccountCommitmentInput | Web3IssuerCommitmentInput;
export declare type Web3IdProofRequest = {
    challenge: string;
    credentialStatements: RequestStatement[];
};
export declare type Web3IdProofInput = {
    request: Web3IdProofRequest;
    globalContext: CryptographicParameters;
    commitmentInputs: CommitmentInput[];
};
export declare type TimestampProperty = {
    title: string;
    type: 'object';
    properties: {
        type: {
            type: 'string';
            const: 'date-time';
        };
        timestamp: {
            type: 'string';
            format?: 'date-time';
        };
    };
    required: ['type', 'timestamp'];
    description?: string;
};
export declare type SimpleProperty = {
    title: string;
    description?: string;
    type: 'string' | 'integer';
    format?: string;
};
export declare type CredentialSchemaProperty = SimpleProperty | TimestampProperty;
declare type IdDetails = {
    title: string;
    description?: string;
    type: 'string';
};
declare type CredentialSchemaAttributes = {
    title?: string;
    description?: string;
    type: 'object';
    properties: Record<string, CredentialSchemaProperty | TimestampProperty>;
    required: string[];
};
export declare type CredentialSchemaSubject = {
    type: 'object';
    properties: {
        id: IdDetails;
        attributes: CredentialSchemaAttributes;
    };
    required: string[];
};
export declare const IDENTITY_SUBJECT_SCHEMA: CredentialSchemaSubject;
export declare type RangeStatementV2 = GenericRangeStatement<string, AttributeType>;
export declare type NonMembershipStatementV2 = GenericNonMembershipStatement<string, AttributeType>;
export declare type MembershipStatementV2 = GenericMembershipStatement<string, AttributeType>;
export declare type RevealStatementV2 = GenericRevealStatement<string>;
export declare type AtomicStatementV2 = GenericAtomicStatement<string, AttributeType>;
export declare type VerifiableCredentialQualifier = {
    type: 'sci';
    issuers: ContractAddress[];
};
declare type IdentityProviderIndex = number;
export declare type IdentityQualifier = {
    type: 'cred';
    issuers: IdentityProviderIndex[];
};
export declare type StatementProverQualifier = VerifiableCredentialQualifier | IdentityQualifier;
export declare function isAccountCredentialStatement(statement: CredentialStatement): statement is AccountCredentialStatement;
export declare function isVerifiableCredentialStatement(statement: CredentialStatement): statement is VerifiableCredentialStatement;
export interface AccountCredentialStatement extends CredentialStatement {
    idQualifier: IdentityQualifier;
    statement: AtomicStatementV2[];
}
export interface VerifiableCredentialStatement extends CredentialStatement {
    idQualifier: VerifiableCredentialQualifier;
    statement: AtomicStatementV2[];
}
export declare type CredentialStatement = {
    idQualifier: StatementProverQualifier;
    statement: AtomicStatementV2[];
};
export declare type RequestStatement = {
    id: string;
    statement: AtomicStatementV2[];
    /** The type field is present iff the request is for a verifiable credential */
    type?: string[];
};
export declare function isVerifiableCredentialRequestStatement(statement: RequestStatement): boolean;
export declare type CredentialStatements = CredentialStatement[];
export declare type CredentialSubject = {
    id: string;
    attributes: Record<string, AttributeType>;
};
export {};
